{----------------------------------------------------------------------------}
{TNCINIT.PAS                                                                 }
{                                                                            }
{Das Programm TNCINIT dient zur automatischen Parametrierung eines TNCs mit  }
{TAPR-SW oder WA8DED-/TF-SW.                                                 }
{                                                                            }
{Dabei kann der Anwender alle Uebertragungsschutzzeiten optimal auf die      }
{Erfordernisse seines TNCs anpassen.                                         }
{Zusaetzlich sind Funktionen wie KISS-Reset,Verlassen des WA8DED-Hostmodus,  }
{Schutzzeit und Uhrzeitsetzen mit den Daten der PC-Uhr ueber spezielle       }
{Befehlszeilen moeglich.                                                     }
{                                                                            }
{Version 9.0 mit UNIT V24 vom 19.06.93  Gerd Michael Ritter DL5FBD           }
{----------------------------------------------------------------------------}

Program TNCINIT;

Uses CRT,DOS,V24;


{----------------------------------------------------------------------------}
{Kennzeichnung der aktuellen Programmversion                                 }
{----------------------------------------------------------------------------}

CONST   Version          ='9.0';


{----------------------------------------------------------------------------}
{Arbeitskonstanten und Arbeitsvariablen von TNCINIT                          }
{----------------------------------------------------------------------------}

CONST   Breakdauer       =100;          {Breakzeit fuer TAPR in ms}

VAR     I               :Byte;          {Indexvariable}
        DATEI           :File of Char;  {Ausgabedatei}
        Dummi           :Integer;       {Hilfsvariable fuer Prozeduren}
        DummiS          :String;        {Hilfsvariable fuer Prozeduren}
        Command         :Boolean;       {Status fuer Komandomodusschaltung TNC}
        WA8DATUM        :STRING;        {Datum WA8DED/TF Europ. Format}
        WA8UHR          :STRING;        {Uhrzeit WA8DED/TF Europ. Format}
        TAPRDATUHR      :STRING;        {Datums-Uhrzeit-Parameter TAPR}
        AUTOSYNC        :BYTE;          {Synczeichencode fuer Autobaudroutinen}
        Zeile           :String;        {Eingabezeile aus Datei}
        Name            :String[80];    {Dateiname der Parameterdatei}
        TNCSoft         :String[6];     {Softwarekennung fuer TNC-SW}
        Mode            :String[3];     {Softwarekennung fuer KISS-Umschaltung}
        ZZP             :Word;          {Zeichen-Zeichen-Pause in ms}
        ZLP             :Word;          {Zeile-Zeile-Pause in ms}
        BNP             :Word;          {Benutzerdefinierbare Pause}
        RXEcho          :Boolean;       {Echoindikator fuer Empfangsdaten}

        COM             :String[10];    {Definitionsstring fuer COM-Port}
        SER             :Byte;          {COM-Nummer der RS232-Schnittstelle}
        Adresse         :Word;          {Basisadresse bei externer HW-Defin.}
        IRQ             :Byte;          {IRQ bei externer HW-Definition}
        HSL             :Byte;          {Handshakeleitungsbitmuster DTR,RTS}
        Baud            :Word;          {Baudrate}
        Bitzahl         :Byte;          {Bitzahl}
        Paritaet        :Char;          {Paritaetskennung 'g','u','k'}
        Stopbit         :Byte;          {Stopbits}
        Handshake       :Char;          {Handshakekennung 'S','H','K'}        
        TXFreigabe      :Boolean;       {Zwischenspeicher fuer Sendehandshake}
        Antwort         :Char;          {Tastendruck bei Bildschirmseiten}
        UTCMin          :ShortInt;      {UTC-Minutenoffset}
        UTCStd          :ShortInt;      {UTC-Stundenoffset}
        WertB           :Byte;          {Hilfsvariablen zur Extrahierung von}
        WertW           :Word;          {numerischen Werten aus den ueberge-}
        WertS           :String;        {benen Zeichenketten}

{****************************************************************************}
{ Proceduren der TNC-Kissrueckschaltung und Reinitialisierung                }
{****************************************************************************} 

{----------------------------------------------------------------------------}
{ Vorhandene Zeichen von RS232 lesen                                         }
{----------------------------------------------------------------------------}

Procedure V24Input;

VAR Test   :Char;

BEGIN
  While V24Count>0 DO                 {Auf RS232 empfangene Zeichen einlesen}
    BEGIN 
      TEST:=V24Read;
      IF RXEcho THEN                  {Nur fuer Testzwecke ueber VAR RXEcho} 
        BEGIN                         {Bildschirmausgabe einschaltbar!}
          IF (Ord(Test) AND 127)<>10 THEN Write (CHR(Ord(Test) AND 127));
          IF (Ord(Test) AND 127)= 13 THEN Writeln;
        END;
    END;
END;


{----------------------------------------------------------------------------}
{ Zeichen unter Sendehandshakebeachtung ausgeben                             }
{----------------------------------------------------------------------------}

PROCEDURE V24HSWrite(Zeichen:CHAR);

BEGIN
  REPEAT
    TXFreigabe:=TXReady;
    IF TXFreigabe THEN
      BEGIN
        V24Write(Zeichen);                {Zeichen auf RS232 ausgeben}
        Delay(ZZP);                       {Zeichenschutzzeit abwarten}
      END;
  UNTIL TXFreigabe;
END;


{----------------------------------------------------------------------------}
{ Zeichenkette auf RS232 ausgeben                                            }
{----------------------------------------------------------------------------}

PROCEDURE V24STWrite(ZKETTE:STRING);

VAR I    :BYTE;

BEGIN
  IF (ZKette<>'') OR (NOT Command) THEN
    BEGIN
      
      IF Command THEN
        {Bei WA8-SW vor jeder Befehlszeile CTRL-X plus ESC senden}
        IF TNCSOFT <> 'TAPR' THEN 
          BEGIN
            V24HSWrite(CHR(24));      {CTRL-X}
            V24HSWrite(CHR(27));      {ESC}
          END
        {Bei TAPR-SW vor jeder Befahlszeile ein BREAK}
        ELSE SendBreak(Breakdauer);

      IF ZKette<>'' THEN FOR I:=1 TO LENGTH(ZKETTE) DO V24HSWrite(ZKETTE[I]);
      V24HSWrite(CHR(13));
      Delay(ZLP);                     {Zeilenschutzzeit abwarten}
    END;
END;


{----------------------------------------------------------------------------}
{ Autobaudsynchronisation                                                    }
{----------------------------------------------------------------------------}

Procedure AUTOBAUD(AUTOSYNC:BYTE);

VAR SYNC    :Boolean;
    Anzahl  :WORD;
    S,Z     :BYTE;
    Test    :CHAR;

BEGIN
  IF AUTOSYNC=0 THEN EXIT;

  Write('*** AUTOBAUD-SYNC Versuch ');
  S:=WhereX;
  Z:=WhereY;
  Anzahl:=0;

  REPEAT

    V24Input;                       {RS232 Puffer auslesen}
    REPEAT
      V24Write(chr(AUTOSYNC));      {SYNC-Zeichen senden bis Zeichen empfangen}
      Delay(ZZP);
      INC(Anzahl);                  {und zwischengespeichert wurden}
      GotoXY(S,Z);
      Write(Anzahl);
    UNTIL V24Count>0;

    Writeln;
    Delay(ZLP);
    V24Input;                       {RS232-Puffer auslesen}
    V24Write(chr(13));              {Auf CR muss CRLF+Kommandoprompt folgen}
    Delay(ZLP);
    IF V24Count>0 THEN              {Zeichen lesen und auf CR oder LF pruefen}
      REPEAT        
        Test:=V24Read;
        SYNC:=( (Test=chr(13)) OR (Test=chr(10)) );
      UNTIL (SYNC) OR (V24Count=0); {Wenn CR oder LF empfangen dann ok}

  UNTIL SYNC;                       {Wenn CR oder LF empfangen dann fertig}
  
  V24Input;                         {RS232-Puffer auslesen}
END;


{----------------------------------------------------------------------------}
{ Trennung von durch Kommas getrennter Parameterliste                        }
{----------------------------------------------------------------------------}

Procedure Trennen(VAR Liste,Parameter:String);

VAR     Posit    :Byte;

BEGIN
  POSIT:=POS(',',Liste);
  IF POSIT>0 THEN
    BEGIN
      Parameter:=COPY(Liste,1,POSIT-1);
      Liste:=COPY(Liste,Posit+1,Length(Liste)-Posit);
    END
  ELSE
    BEGIN
      Parameter:=Liste;
      Liste:='';
    END;
END;


{----------------------------------------------------------------------------}
{ Datum+Uhrzeit konstruieren und zum TNC uebertragen                         }
{----------------------------------------------------------------------------}

PROCEDURE DATUMUHR(Software:String;UTCStd:ShortInt;UTCMin:ShortInt);

VAR     Jahr        :Word;
        Monat       :Word;
        Tag         :Word;
        Stunden     :Word;
        Minuten     :Word;
        Sekunden    :Word;
        TAPRDatUhr  :String[12];
        WA8DEDDatum :String[8];
        WA8DEDZeit  :String[8];
        Dummi       :Word;  {Hilfsvariable fuer diverse Zwecke}


Procedure DatumUhrzeit(UTCStd:ShortInt;UTCMin:ShortInt);

CONST Schaltjahr=1992;
      MTAGE: Array[1..12] of Byte =(31,28,31,30,31,30,31,31,30,31,30,31);
      SpT  : LongInt              =86340;      

VAR Offset : LongInt;
    Zeit_S : LongInt;
    RW     : Integer;


Function Zeit_in_Sekunden(Stunden,Minuten,Sekunden:LongInt):LongInt;

BEGIN
  Zeit_in_Sekunden:=Stunden*3600+Minuten*60+Sekunden;
END;


BEGIN
  
  GetTime(Stunden,Minuten,Sekunden,Dummi);
  Sekunden:=Sekunden+Round(Dummi/100+0.5);
  Zeit_S:=Zeit_in_Sekunden(Stunden,Minuten,Sekunden);
  Offset:=Zeit_in_Sekunden(UTCStd,UTCMin,0);
  Zeit_S:=Zeit_S+Offset;
  GetDate(Jahr,Monat,Tag,Dummi);
  IF ((Jahr-SchaltJahr) DIV 4)*4 = (Jahr-Schaltjahr) THEN MTAGE[2]:=29;

  IF Zeit_S<0 THEN
    BEGIN
      Zeit_S:=Zeit_S+SpT;
      IF (Tag=1) AND (Monat=1) THEN
        BEGIN
          DEC(Jahr);
          Monat:=12;
          Tag:=31;
        END
      ELSE IF Tag=1 THEN
        BEGIN
          DEC(Monat);
          Tag:=MTAGE[Monat];
        END
      ELSE DEC(Tag);
    END;

  IF Zeit_S>SPT THEN
    BEGIN
      Zeit_S:=Zeit_S-SpT;
      IF (Tag=31) AND (Monat=12) THEN
        BEGIN
          INC(Jahr);
          Monat:=1;
          Tag:=1;
        END
      ELSE IF Tag=MTage[Monat] THEN
        BEGIN
          Tag:=1;
          INC(Monat);
        END
      ELSE INC(Tag);
    END;

  Stunden:=Zeit_S DIV 3600;
  RW:=Zeit_S-Stunden*3600;
  Minuten:=RW DIV 60;
  Sekunden:=RW-Minuten*60;
END;


Function STW(Nummer:WORD):String;

VAR     HS      :String;

BEGIN
  STR(Nummer,HS);
  IF Nummer > 9 THEN STW:=HS            {Zweistelliges Format garantieren}
                ELSE STW:='0'+HS;       {1-9 werden 01-09}
END;


BEGIN
  DatumUhrzeit(UTCStd,UTCMin);
  TAPRDatUhr :=STW(Jahr-1900)+STW(Monat)+STW(Tag)+STW(Stunden)+STW(Minuten)
              +STW(Sekunden);
  WA8DEDDatum:=STW(TAG)+'.'+STW(Monat)+'.'+STW(Jahr-1900);
  WA8DEDZeit :=STW(Stunden)+':'+STW(Minuten)+':'+STW(Sekunden);

  {Datum und Uhrzeit setzen}
  IF Software='TAPR' THEN V24STWRITE('DA '+TAPRDATUHR)
                     ELSE 
                       BEGIN
                         V24STWRITE('K '+WA8DEDDatum);
                         V24STWRITE('K '+WA8DEDZeit);
                       END;

  IF RXEcho THEN Writeln(#10);  {Formattierung bei Empfangsecho}
  Writeln('*** Uhrzeit und Datum (aus PC-Systemuhr) wurden gesetzt.');
END;


{----------------------------------------------------------------------------}
{ Zeichenkette aus Char-Datei lesen                                          }
{----------------------------------------------------------------------------}

Procedure READSTR(VAR DATZEI:String);

VAR Zeich       :CHAR;

BEGIN
  DATZEI:='';
  REPEAT
    READ (Datei,Zeich);
    IF (    (Ord(Zeich)<>10) 
        AND (Ord(Zeich)<>13)
        AND (Ord(Zeich)<>26)    ) THEN DATZEI:=DATZEI+Zeich;
  UNTIL EOF(Datei) OR (Ord(Zeich)=10);
END; 
    

{----------------------------------------------------------------------------}
{ KISS-Modus ausschalten                                                     }
{----------------------------------------------------------------------------}

Procedure KISSOFF;

BEGIN
  Write('*** Kiss-Reset-Befehl wird uebertragen - ');
  V24HSWrite(chr(192));
  V24HSWrite(chr(255));
  {KISS-Ruecksetz-Befehl: TF2.X <FEND><255> und TAPR <FEND><255><FEND>}
  IF TNCSOFT='TAPR' THEN  V24HSWrite(chr(192));
  Writeln('ok!');
END;


{----------------------------------------------------------------------------}
{ Hostmodus der WA8DED-Software oder The Firmware ausschalten                }
{----------------------------------------------------------------------------}

Procedure HostOff;

VAR JHost0 :String[9];
    Muell  :Char;

BEGIN
  Write('*** WA8DED-Hostmode wird ausgeschaltet - ');
  REPEAT
    V24HSWrite(#1);
    Delay(ZZP);
  UNTIL V24Count>0;
  Delay(ZLP);
  While V24Count>0 DO Muell:=V24Read;
  JHost0:=#1+#1+#5+'JHOST0';
  FOR I:=1 TO 9 DO
    BEGIN
      V24HSWrite(JHost0[I]);
      Delay(ZZP);
    END;
  Writeln('ok!');
  Delay(ZLP);
  Muell:=V24Read;
END;


{----------------------------------------------------------------------------}
{ Befehlsanalyse der Kommandozeilenparameter                                 }
{----------------------------------------------------------------------------}

Procedure PARAAUSW(Test:String);

VAR     Dummi   :Integer;  {Pruefvariable fuer Wandlung String->Ziffer}
        I       :Byte;
        Posit   :Byte;
        Laenge  :Byte;

BEGIN
  Laenge:=Length(Test);

  {Einheitliche Umwandlung in Grossbuchstaben}
  FOR I:=1 TO Length(Test) DO Test[I]:=UpCase(Test[I]);   

  {Extrahierung Filenamen}
  Posit:=POS('F:',Test);
  IF Posit=1 THEN
    BEGIN
      Name:=COPY (Test,Posit+2,(Laenge-2));
      Exit;
    END;

  {Extrahierung Schnittstellendefinition}
  Posit:=POS('C:',Test);
  IF Posit=1 THEN 
    BEGIN
      COM:=COPY (Test,Posit+2,(Laenge-2));
      Exit;
    END;

  {Extrahierung Baudrate}
  Posit:=POS('B:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+2,(Laenge-2))),Baud,Dummi);
      IF Dummi<>0 THEN Baud:=9600;
      Exit;
    END;

  {Extrahierung Wortlaenge}
  Posit:=POS('W:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+2,(Laenge-2))),Bitzahl,Dummi);
      IF Dummi<>0 THEN Bitzahl:=8;
      Exit;
    END;

  {Extrahierung Paritaet}
  Posit:=POS('P:',Test);
  IF Posit=1 THEN 
    BEGIN
      Paritaet:=Test[3];
      IF NOT (Paritaet IN ['G','U','K'])  THEN Paritaet:='K'; 
      Exit;
    END;

  {Extrahierung Stopbits}
  Posit:=POS('SB:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+3,(Laenge-3))),Stopbit,Dummi);
      IF Dummi<>0 THEN Stopbit:=1;
      Exit;
    END;

  {Extrahierung Handshakeverfahren}
  Posit:=POS('H:',Test);
  IF Posit=1 THEN 
    BEGIN
      Handshake:=Test[3];
      IF NOT (Paritaet IN ['H','S','K'])  THEN Handshake:='K'; 
      Exit;
    END;

  {Extrahierung TNC-SW}
  Posit:=POS('S:',Test);
  IF Posit=1 THEN
    BEGIN
      TNCSoft:=COPY (Test,Posit+2,(Laenge-2));
      Exit;
    END;

  {Extrahierung Terminalechoausgabe}
  Posit:=POS('E:',Test);
  IF Posit=1 THEN
    BEGIN
      RXEcho:=TRUE;
      Exit;
    END;

  {Extrahierung UTC-Offset fuer Uhr und Datum}
  Posit:=POS('U:',Test);
  IF (Posit=1) AND (Length(Test)>2) THEN 
    BEGIN
      Test:=COPY (Test,Posit+2,(Laenge-2));
      Trennen(Test,WertS);
      IF WertS<>'' THEN
        BEGIN
          VAL(WertS,UTCStd,Dummi);
          IF Dummi>0 THEN 
            BEGIN
              UTCStd:=0;
              EXIT;
            END;
        END;
      Trennen(Test,WertS);
      IF WertS<>'' THEN
        BEGIN
          VAL(WertS,UTCMin,Dummi);
          IF Dummi>0 THEN 
            BEGIN
              UTCMin:=0;
              EXIT;
            END;
        END;
      Exit;
    END;

END;


{----------------------------------------------------------------------------}
{ Auswertung der uebergebenen Kommandozeilenparameter                        }
{----------------------------------------------------------------------------}

Procedure PARAANAL;

VAR     I       :Byte;
        
BEGIN
  For I:=1 TO ParamCount DO PARAAUSW(ParamStr(I));
END;


{----------------------------------------------------------------------------}
{ Auswertung von Dateibefehlen in der Parameterdatei                         }
{----------------------------------------------------------------------------}

Procedure KOMMAUSW(Test:String);

VAR     Dummi   :Integer;  {Pruefvariable fuer Wandlung String->Ziffer}
        I       :Byte;
        Posit   :Byte;
        Laenge  :Byte;

BEGIN

  {Trennung von Nutzdaten und Kommentaren}
  Posit:=POS(' ',TEST);         {Trennzeichen= ' '}
  Laenge:=Posit-1;
  IF Posit>1 THEN Test:=COPY(Test,1,Laenge);
  Posit:=POS(chr(9),TEST);      {Trennzeichen= TAB}
  Laenge:=Posit-1;
  IF Posit>1 THEN Test:=COPY(Test,1,Laenge);

  {Einheitliche Umwandlung in Grossbuchstaben}
  FOR I:=1 TO Length(Test) DO Test[I]:=UpCase(Test[I]);   

  {Extrahierung Terminalechoausgabe}
  Posit:=POS('E:',Test);
  IF Posit=1 THEN 
    BEGIN
      Test:=COPY (Test,Posit+2,(Laenge-2));
      RXEcho:=(Test='ON');
      Exit;
    END;

  {Extrahierung Autobaudsynczeichencode und Autobaudsynchronisierung}
  Posit:=POS('A:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+2,(Laenge-2))),AUTOSYNC,Dummi);
      IF Dummi=0 THEN AUTOBAUD(AUTOSYNC);
      Exit;
    END;

  {Extrahierung und Ausfuehrung KISS-Ruecksetzen}
  Posit:=POS('K:',Test);
  IF Posit=1 THEN
    BEGIN
      KISSOFF;
      Exit;
    END;

  {Extrahierung und Ausfuehrung Uhr-Stellen}
  Posit:=POS('U:',Test);
  IF (Posit=1) AND (Length(Test)>2) THEN 
    BEGIN
      Test:=COPY (Test,Posit+2,(Laenge-2));
      Trennen(Test,WertS);
      IF WertS<>'' THEN
        BEGIN
          VAL(WertS,UTCStd,Dummi);
          IF Dummi>0 THEN 
            BEGIN
              UTCStd:=0;
              EXIT;
            END;
        END;
      Trennen(Test,WertS);
      IF WertS<>'' THEN
        BEGIN
          VAL(WertS,UTCMin,Dummi);
          IF Dummi>0 THEN 
            BEGIN
              UTCMin:=0;
              EXIT;
            END;
        END;
      DatumUhr(TNCSoft,UTCStd,UTCMin);
      EXIT;
    END
  ELSE IF Posit=1 THEN
    BEGIN
      DatumUhr(TNCSoft,UTCStd,UTCMin);
      EXIT;
    END;

  {Extrahierung Kommandomoduseinstellung des TNC vor jeder Textzeile}
  Posit:=POS('CL:',Test);
  IF Posit=1 THEN 
    BEGIN
      Test:=COPY (Test,Posit+3,(Laenge-3));
      IF Test='ON'  THEN Command:=True;
      IF Test='OFF' THEN Command:=False;
      EXIT;
    END;

  {Extrahierung Hostmodeabschaltung WA8DED-Software}
  Posit:=POS('HR:',Test);
  IF Posit=1 THEN
    BEGIN
      IF TNCSoft<>'TAPR' THEN HostOff;
      Exit;
    END;

  {Extrahierung und Ausfuehren BREAK-Interrupt}
  Posit:=POS('BR:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+3,(Laenge-3))),WertW,Dummi);
      IF Dummi<>0 THEN WertW:=Breakdauer;
      SendBreak(WertW);
      Exit;
    END;

  {Extrahierung und Aussenden von Byte-Zeichencodes}
  Posit:=POS('BY:',Test);
  IF Posit=1 THEN 
    BEGIN
      Test:=COPY (Test,Posit+3,(Laenge-3));
      REPEAT
        Trennen(Test,WertS);
        VAL(WertS,WertB,Dummi);
        IF Dummi=0 THEN V24HSWrite(chr(WertB));
      UNTIL Test='';
      Exit;
    END;

  {Extrahierung Zeichenschutzzeit}
  Posit:=POS('CD:',Test);
  IF Posit=1 THEN
    BEGIN
      VAL((COPY (Test,Posit+3,(Laenge-3))),ZZP,Dummi);
      IF Dummi<>0 THEN ZZP:=25;
      Exit;
    END;

  {Extrahierung Zeilenschutzzeit}
  Posit:=POS('LD:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+3,(Laenge-3))),ZLP,Dummi);
      IF Dummi<>0 THEN ZLP:=300;
      Exit;
    END;

  {Extrahierung und Abwarten der Benutzer-Schutzzeit}
  Posit:=POS('D:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+2,(Laenge-2))),BNP,Dummi);
      IF Dummi<>0 THEN BNP:=1000;
      Delay(BNP);
      Exit;
    END;

  {Extrahierung des DTR,RTS-Rasterbits fuer TNCIO und Einstellung}
  Posit:=POS('L:',Test);
  IF Posit=1 THEN 
    BEGIN
      VAL((COPY (Test,Posit+2,(Laenge-2))),HSL,Dummi);
      IF Dummi<>0 THEN HSL:=0;
      CASE HSL OF
        0: BEGIN
             ClearDTR;
             ClearRTS;
           END;
        1: BEGIN
             ClearDTR;
             SetRTS;
           END;
        2: BEGIN
             SetDTR;
             ClearRTS;
           END;
        3: BEGIN
             SetDTR;
             SetRTS;
           END;
      END;
      Exit;
    END;

END;


{----------------------------------------------------------------------------}
{ Pause bei Bildschirmausgaben                                               }
{----------------------------------------------------------------------------}

Procedure PAUSE;

BEGIN
  Write('                 *** Weiter mit beliebiger Taste ***');
  REPEAT UNTIL KEYPRESSED;
  Antwort:=ReadKey;
END;


{****************************************************************************}
{ Hauptprogramm                                                              }
{****************************************************************************}

BEGIN

  {Standardparameter vordefinieren}
  Name:='';
  COM:='1';
  BAUD:=9600;
  Handshake:='K';
  Bitzahl:=8;
  Paritaet:='K';
  Stopbit:=1;
  AUTOSYNC:=0;
  TNCSoft:='TAPR';
  Mode:='OFF';
  Command:=True;
  RXEcho:=False;
  ZZP:=25;
  ZLP:=300;
  BNP:=1000;
  UTCStd:=0;
  UTCMin:=0;

  ClrScr;

  {Kommandozeilenparameter analysieren}
  ParaAnal;

  IF (ParamCount=0) OR (Name='') THEN
    BEGIN
      Writeln;
      Writeln(' TNCINIT     Kissmodusumschaltung oder Neukonfiguration einer');
      Writeln('  V ',Version,'      TAPR oder WA8DED/TF-SW mittels Initialisierungsdatei.');
      Writeln;
      Writeln('(c)DL5FBD');
      Writeln;
      Writeln('  Aufruf:       TNCINIT Parameterliste');
      Writeln;
      Writeln('Die Parameter bestehen aus einer Kennung mit ":" als Trennung.');
      Writeln('Der Dateiname muss angegeben werden, alle anderen Parameter');
      Writeln('sind mit Standardvorgaben siehe Werte in (...) vordefiniert.'); 
      Writeln;
      Writeln(' F:Datei        Name der Initialisierungsdatei');
      Writeln(' C:Port         Serielle Schnittstelle COMX         (1)');
      Writeln(' C:Port,Adr,IRQ vollstaendige HW-Definition');
      Writeln(' B:Baudrate     Baudrate                            (9600)');
      Writeln(' W:Wortlaenge   Wortlaenge 7,8                      (8)');
      Writeln(' P:Paritaet     Paritaet Keine,Gerade,Ungerade      (K)');
      Writeln(' SB:Stopbit     Stopbits 1,2                        (1)');
      Writeln(' H:Handshake    Handshake Keines,Hardware,Software  (K)');
      Writeln(' S:TNCSoft      Softwarekennung "TAPR","TF"         (TAPR)');
      Writeln(' E:             Empfangsdatenecho auf Bildschirm    (kein Echo)');
      Writeln(' U:ss,mm        UTC-Zeitoffset in Stunden,Minuten   (0,0)');
      Writeln;
      Pause;
      ClrScr;
      Writeln;
      Writeln('In der Parameterdatei knnnen Befehlszeilen eingefgt werden.');
      Writeln('Mit diesen Befehlszeilen werden TNCINIT-Aktionen angefordert,');
      Writeln('Schutzeiten umdefiniert oder ausgefhrt.');
      Writeln;
      Writeln(' D:Zeit (ms)     Allgemeine Schutzzeit in ms');
      Writeln('                 Diese Schutzzeit kann an beliebiger Stelle in');
      Writeln('                 der Parameterdatei angefordert werden und wird');
      Writeln('                 unmittelbar abgewartet.');
      Writeln(' CD:Zeit (ms)    Zeichenschutzzeit in ms');
      Writeln('                 Diese Schutzzeit wird zwischen dem Aussenden');
      Writeln('                 aufeinanderfolgender Zeichen eingehalten.');
      Writeln(' LD:Zeit (ms)    Zeilenschutzzeit in ms');
      Writeln('                 Diese Schutzzeit wird am Ende einer uebertragenem');
      Writeln('                 Zeile ausgefuehrt und gibt dem TNC zusaetzliche');
      Writeln('                 Zeit zum interpretieren und ausfuehren.');
      Writeln(' K:              Kiss Ruecksetzkommando');
      Writeln('                 Fuehrt das KISS-Ruecksetzkommando abhaengig von');
      Writeln('                 der WA8DED/TF-SW oder TAPR-SW aus.');
      Writeln(' U:              Uhrzeit und Datum uebertragen.');
      Writeln(' U:mm,ss         Uhrzeit mit Offset (Std,Min) fuer UTC');
      Writeln('                 Uebertraegt die aktuelle PC-Uhrzeit zum TNC.');        
      Writeln(' E:ON E:OFF      Schaltet Echo ein "ON" oder aus "OFF"');
      Writeln('                 Dient zum Austesten der Schutzzeitparameter.');
      Writeln;
      Pause;
      ClrScr;
      Writeln;
      Writeln('BR:Zeit (ms)     Sendet fuer die angegebene Zeit in ms ein Break');
      Writeln('                 Signal aus.');     
      Writeln('BY:xx,yy,zz,..   Sendet eine Folge von Bytes aus die Dezimal oder');
      Writeln('                 Hexadezimel ($n) angegeben und duch Kommas ge-');
      Writeln('                 trennt sind. Damit ist es moeglich auch CTRL-');
      Writeln('                 zeichen zu uebertragen.');
      Writeln('CL:ON CL:OFF     CL:ON bewirkt das vor der Uebertragung einer Zeile');
      Writeln('                 der TNC in den Kommandomodus umgeschaltet wird.');
      Writeln('                 CL:OFF schaltet diese Umschaltung aus, z.B. zum');
      Writeln('                 Initialisieren von TNC-Mailboxen (PK232 oae.)');
      Writeln('HR:              Schaltet einen TNC mit WA8DED/TF-Software aus dem');
      Writeln('                 Hostmodus in den Terminalmodus um.');
      Writeln('L:xx             Gezieltes Setzen/Ruecksetzen der Handshakelei-');
      Writeln('                 tungen RTS,DTR ueber ein Kontrollbyte im Format');
      Writeln('                 DTR= Bit 1 und RTS= Bit 0 fuer RS232-Multiplex-');
      Writeln('                 hardware wie z.B. die TNCIO-Karte');
      Writeln('                 Je nach Hardware muss der Zusammenhang xx zur');
      Writeln('                 TNC-Nummer am Multiplexer ausprobiert werden.');
      Writeln;
      Pause;
      ClrScr;
      Writeln(': Bemerkung      Einfuegen einer Bemerkungszeile zur Dokumentation.');
      Writeln('                 Bemerkungszeilen werden vom Kommandointerpreter');
      Writeln('                 von TNCINIT anhand des einzelnen (:) am Anfang');
      Writeln('                 der Zeile erkannt.');
      Writeln;
      Writeln('Unabhaengig davon kann an die hier aufgefuehrten TNCINIT-Befehle');
      Writeln('durch mindestens ein Leerzeichen oder Tabulatorzeichen getrennt');

      Writeln('ein Kommentar angefuegt werden.');
      Writeln;
      Writeln('z.B.   CD:25     25ms  Zeichenschutzzeit');
      Writeln('       LD:200    200ms Zeilenschutzzeit');
      Writeln;
      Writeln('Diese Kommentare machen eine Initialisierungsdatei wesentlich');
      Writeln('verstaendlicher und nachvollziehbarer.');
      Writeln;
      Pause;
      ClrScr;
      Writeln;
      Writeln('Aufrufbeispiel fuer COM3 Adresse=338H IRQ=2 mit 19200 Baud');
      Writeln('und der Initialisierungsdatei KISSOFF.DAT fuer TAPR-SW.');
      Writeln;
      Writeln('TNCINIT C:3,$338,2 B:19200 S:TAPR F:KISSOFF.DAT');
      Writeln;
      Writeln('Die Initialisierungsdatei KISSOFF.DAT kann wie folgt aussehen:');
      Writeln;
      Writeln('   : Demonstration einer KISS-Reset und Initialisierungsdatei');
      Writeln('   CD:20            Zeichenschutzzeit 20ms');
      Writeln('   LD:100           Befehlsschutzzeit 100ms');
      Writeln('   K:               KISS-Reset senden');
      Writeln('   D:2000           Nach KISS-Reset 2s Initialisierungspause');
      Writeln('   AWLEN 8');
      Writeln('   PARITY 0');
      Writeln('   MYCALL DL5FBD');
      Writeln('   RESTART');
      Writeln('   D:2000           Nach RESTART 2s Initialisierungspause');
      Writeln;
      Writeln('Die Uebertragungsparameter sind hier: 8 Bit, keine Paritaet und');
      Writeln('1 Stopbit die standardmaessig in TNCINIT vordefiniert sind.');
      Writeln;
      Halt;
    END;

  {Schnittstellenstring auswerten und Schnittstellennummer,Basisadresse}
  {und den IRQ bei erweiterter Hardwaredefinition entnehmen.}

  {Einfache Angabe der COM-Nummer bei COM1 oder COM2}
  IF LENGTH(COM)=1 THEN VAL(COPY(COM,1,1),SER,Dummi)
  {Erweiterte Definition von COMX,Basisadresse,IRQ fuer COM3 oder COM4}
  ELSE 
    BEGIN
      DummiS:=COM;
      Trennen(DummiS,WertS);               {COMx-Nr. entnehmen}
      VAL(WertS,SER,Dummi);
      IF Dummi=0 THEN
        BEGIN
          Trennen(DummiS,WertS);           {Basisadresse entnehmen}
          VAL(WertS,Adresse,Dummi);
        END;
      IF Dummi=0 THEN
        BEGIN
          Trennen(DummiS,WertS);           {IRQ entnehmen}
          VAL(WertS,IRQ,Dummi);
        END;
    END;
  {Bei Extrahierungsfehler(n) von SER, Adresse oder IRQ wird abgebrochen}
  IF Dummi<>0 THEN
    BEGIN
      Writeln;
      Write('*** TNCINIT ',Version,' Abbruch COM-Port-Angabe: ',COM);
      Writeln(' nicht auswertbar!');
      Writeln;
      halt;
    END;
  {Externe HW-Definitionen in der UNIT V24 einstellen}
  IF LENGTH(COM)>=8 THEN
    BEGIN
      IF SER<>5 THEN V24DefHW(SER,Adresse,IRQ)
      ELSE
        BEGIN
          Writeln;
          Write('*** TNCINIT ',Version,' Abbruch TFPCx Softwareinterface: ',COM);
          Writeln(' ist keine Hardware!');
          Writeln;
          halt;
        END;
    END;

  {Initialisierungsdatei zum Lesen oeffnen}
  ASSIGN(Datei,Name);
  {$I-} Reset(Datei); {$I+}
  If IOResult<>0 THEN
    BEGIN
      Writeln;
      Writeln('*** TNCINIT ',Version,' -Abbruch: ',Name,' nicht gefunden!');
      Writeln;
      Halt;
    END;

  {Serielle Schnittstelle initialisieren sofern vorhanden}
  IF SER>4 THEN
    BEGIN
      Writeln;
      Writeln('*** TNCINIT ',Version,' -Abbruch: COM=',SER,' Erlaubte Ports sind 1-4');
      Writeln;
      Halt;
    END;

  IF V24Install(SER) THEN V24OPEN(SER,Baud,Bitzahl,Paritaet,Stopbit,Handshake)
  ELSE
    BEGIN
      Writeln;
      Writeln('*** TNCINIT ',Version,' -Abbruch: COM',SER,' ist nicht installiert!');
      Writeln;
      Halt;
    END;
          
  {Ausgabesteuerung }
  Writeln;
  Writeln('TNCINIT ',Version,' TNC-(KISS)-Konfigurationsprogramm von DL5FBD');
  Writeln;
  Writeln(' Initialisierungsdatei: ',Name);
  Writeln('          TNC-Software: ',TNCSOFT);
  Writeln('         Schnittstelle: COM',COM);
  Writeln('              Baudrate: ',Baud,' Baud');
  Writeln('            Wortlaenge: ',Bitzahl,'Bit');
  Writeln('              Paritaet: ',Paritaet,' K=keine U=ungerade G=gerade');
  Writeln('              Stopbits: ',Stopbit,'Stopbit');
  Writeln('    Handshakeverfahren: ',Handshake,' K=Kein H=Hardware S=Software');
  Writeln; 

  {Initialisierungsdatei auf serielle Schnittstelle ausgeben}  
  Writeln('Die TNC-Kommandos der Parameterdatei werden uebertragen');
  Writeln('und die TNCINIT-Befehle ausgefuehrt:');
  Writeln;

  REPEAT
    ReadStr (Zeile);
    IF (POS(':',Zeile)>3) OR (POS(':',Zeile)=0) THEN V24STWrite(Zeile)
                                                ELSE KOMMAUSW(ZEILE);
    V24Input;
  UNTIL EOF(Datei);

  Close (Datei);

  V24Close;
END.
